@@ -5,7 +5,8 @@ import os
 import sys
 
 from ICCProfile import ICCProfile
-from config import get_data_path, get_verified_path, getcfg, hascfg, setcfg
+from config import (get_data_path, get_verified_path, getcfg, hascfg,
+					profile_ext, setcfg)
 from log import safe_print
 from meta import name as appname
 from options import debug
@@ -495,11 +496,11 @@ class SynthICCFrame(BaseFrame):
 			# SMPTE 240M
 			self.trc_ctrl.SetSelection(5)
 		elif gamma == -2084:
-			# SMPTE 2084
-			self.trc_ctrl.SetSelection(6)
+			# SMPTE 2084, roll-off clip
+			self.trc_ctrl.SetSelection(7)
 		elif gamma == -2.4:
 			# sRGB
-			self.trc_ctrl.SetSelection(7)
+			self.trc_ctrl.SetSelection(8)
 		else:
 			# Gamma
 			self.trc_ctrl.SetSelection(0)
@@ -524,11 +525,11 @@ class SynthICCFrame(BaseFrame):
 			gamma = 2.2
 			self.trc_gamma_ctrl.Value = str(gamma)
 		white = XYZ["wX"], XYZ["wY"], XYZ["wZ"]
-		if self.trc_ctrl.GetSelection() in (0, 1, 4, 6):
+		if self.trc_ctrl.GetSelection() in (0, 1, 4, 6, 7):
 			# 0 = Gamma
 			# 1 = DICOM - trc set here is not actually used in the end
 			# 4 = Rec. 1886 - trc set here is only used if black = 0
-			# 6 = SMPTE 2084 - trc set here is not actually used in the end
+			# 6 & 7 = SMPTE 2084 - trc set here is not actually used in the end
 			trc = gamma
 		elif self.trc_ctrl.GetSelection() == 2:
 			# L*
@@ -539,7 +540,7 @@ class SynthICCFrame(BaseFrame):
 		elif self.trc_ctrl.GetSelection() == 5:
 			# SMPTE 240M
 			trc = -240
-		elif self.trc_ctrl.GetSelection() == 7:
+		elif self.trc_ctrl.GetSelection() == 8:
 			# sRGB
 			trc = -2.4
 		defaultDir, defaultFile = get_verified_path("last_icc_path")
@@ -598,13 +599,14 @@ class SynthICCFrame(BaseFrame):
 				# Grayscale profile
 				profile.tags.kTRC.set_bt1886_trc(black[1], outoffset, gamma,
 												 getcfg("synthprofile.trc_gamma_type"))
-		elif self.trc_ctrl.GetSelection() == 6:
+		elif self.trc_ctrl.GetSelection() in (6, 7):
 			# SMPTE 2084
 			if self.colorspace_rgb_ctrl.Value:
 				# Color profile
 				profile.set_smpte2084_trc([v * getcfg("synthprofile.luminance")
 										   for v in black],
-										  getcfg("synthprofile.luminance"))
+										  getcfg("synthprofile.luminance"),
+										  rolloff=self.trc_ctrl.GetSelection() == 7)
 			else:
 				# Grayscale profile
 				profile.tags.kTRC.set_smpte2084_trc(getcfg("synthprofile.black_luminance"),
@@ -612,27 +614,23 @@ class SynthICCFrame(BaseFrame):
 		elif black != [0, 0, 0]:
 			if self.colorspace_rgb_ctrl.Value:
 				# Color profile
-				rXYZ = profile.tags.rXYZ.values()
-				gXYZ = profile.tags.gXYZ.values()
-				bXYZ = profile.tags.bXYZ.values()
-				mtx = colormath.Matrix3x3([[rXYZ[0], gXYZ[0], bXYZ[0]],
-										   [rXYZ[1], gXYZ[1], bXYZ[1]],
-										   [rXYZ[2], gXYZ[2], bXYZ[2]]])
-				rgbblack = mtx.inverted() * black
+				vmin = 0
 			else:
 				# Grayscale profile
-				rgbblack = black
-			# Optimize for uInt16Number encoding
-			rgbblack = [round(max(v, 0) * 65535) / 65535 for v in rgbblack]
+				vmin = black[1]
 			for i, channel in enumerate(channels):
 				TRC = profile.tags["%sTRC" % channel]
-				TRC.set_trc(trc, 1024, vmin=rgbblack[i] * 65535)
+				TRC.set_trc(trc, 1024, vmin=vmin * 65535)
+			if self.colorspace_rgb_ctrl.Value:
+				profile.apply_black_offset(black)
 		else:
 			for channel in channels:
 				profile.tags["%sTRC" % channel].set_trc(trc, 1)
 		if black != [0, 0, 0] and self.bpc_ctrl.Value:
-			for channel in channels:
-				profile.tags["%sTRC" % channel].apply_bpc()
+			if self.colorspace_rgb_ctrl.Value:
+				profile.apply_black_offset((0, 0, 0))
+			else:
+				profile.tags.kTRC.apply_bpc()
 		for tagname in ("lumi", "bkpt"):
 			if tagname == "lumi":
 				# Absolute
@@ -663,7 +661,8 @@ class SynthICCFrame(BaseFrame):
 							lang.getstr("save_as"),
 							defaultDir=defaultDir,
 							defaultFile=defaultFile,
-							wildcard="*.icc", 
+							wildcard=lang.getstr("filetype.icc") + 
+									 "|*" + profile_ext, 
 							style=wx.SAVE | wx.FD_OVERWRITE_PROMPT)
 		dlg.Center(wx.BOTH)
 		if dlg.ShowModal() == wx.ID_OK:
@@ -671,7 +670,7 @@ class SynthICCFrame(BaseFrame):
 		dlg.Destroy()
 		if path:
 			if os.path.splitext(path)[1].lower() not in (".icc", ".icm"):
-				path += ".icc"
+				path += profile_ext
 			if not waccess(path, os.W_OK):
 				show_result_dialog(Error(lang.getstr("error.access_denied.write",
 													 path)),
